﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectAtelier.Entities;
using ProjectAtelier.Repositories;
using ProjectAtelier.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAtelier.REPOSITORY.Implementations;

public class MaterialReplenishmentRepository : IMaterialReplenishmentRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<Material> _logger;
    public MaterialReplenishmentRepository(IConnectionString connectionString, ILogger<Material> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateMaterialSpent(MaterialReplenishment material)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(material));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
                INSERT INTO MaterialReplenishment (Count, DataTime, Name)
            VALUES (@Count, @DataTime, @Name)";
            connection.Execute(queryInsert, material);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public MaterialReplenishment ReadMaterialSpentById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
            SELECT * FROM MaterialReplenishment
            WHERE Id=@id";
            var material = connection.QueryFirst<MaterialReplenishment>(querySelect, new
            {
                id
            });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(material));
            return material;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }
    public IEnumerable<MaterialReplenishment> ReadMaterialsSpent(DateTime? dateForm = null, DateTime? dateTo = null, string? nameMaterial = null)
    {
        var builder = new QueryBuilder();
        if (dateForm.HasValue)
        {
            builder.AddCondition("DataTime >= @dateForm");
        }
        if (dateTo.HasValue)
        {
            builder.AddCondition("DataTime <= @dateTo");
        }
        if (nameMaterial != null)
        {
            builder.AddCondition("Name = @nameMaterial");
        }

        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @$"
                SELECT * FROM MaterialReplenishment
                {builder.Build()}";
            var materials = connection.Query<MaterialReplenishment>(querySelect, new { dateForm, dateTo, nameMaterial });
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(materials));
            return materials;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public void UpdateMaterialSpent(MaterialReplenishment material)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(material));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
            UPDATE MaterialReplenishment
            SET
            Count=@Count,
            DataTime=@DataTime,
            Name=@Name
            WHERE Id=@Id";
            connection.Execute(queryUpdate, material);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
}