﻿using Microsoft.Extensions.Logging;
using ProjectAtelier.Repositories;
using ProjectAtelier.REPOSITORY;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAtelier.Reports;

internal class TableReport
{
    private readonly IMaterialReplenishmentRepository _materialReplenishmentRepository;
    private readonly IMaterialRepository _materialRepository;
    private readonly IOrderMaterialsRepository _orderMaterialsRepository;
    private readonly ILogger<TableReport> _logger;
    internal static readonly string[] item = ["Дата", "Количество пришло", "Количество ушло"];

    public TableReport(IMaterialReplenishmentRepository materialrepRepository,
                        IMaterialRepository materialRepository, IOrderMaterialsRepository orderMaterialsRepository, ILogger<TableReport> logger)
    {
        _materialReplenishmentRepository = materialrepRepository ?? throw new ArgumentNullException(nameof(materialrepRepository));
        _materialRepository = materialRepository ?? throw new ArgumentNullException(nameof(materialRepository));
        _orderMaterialsRepository = orderMaterialsRepository ?? throw new ArgumentNullException(nameof(materialRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateTable(string filePath, int materialId, DateTime startDate, DateTime endDate)
    {
        try
        {
            new ExcelBuilder(filePath)
                .AddHeader("Сводка по движению материала", 0, 3)
               .AddParagraph($"за период c {startDate:dd.MM.yyyy} по {endDate:dd.MM.yyyy}", 0)
                .AddTable([10, 15, 15], GetData(materialId, startDate, endDate))
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetData(int materialId, DateTime startDate, DateTime endDate)
    {
        var data = _materialReplenishmentRepository
            .ReadMaterialsSpent(dateForm: startDate, dateTo: endDate, nameMaterial: _materialRepository.ReadMaterialById(materialId).Name)
            .Select(x => new { Date = x.DataTime, CountIn = (int?)x.Count, CountOut = (int?)null })
            .Union(
                _orderMaterialsRepository
                    .ReadOrders(dateForm: startDate, dateTo: endDate, matid: materialId)
                    .Select(x => new { Date = x.DataOrder, CountIn = (int?)null, CountOut = (int?)x.Count }))
            .OrderBy(x => x.Date);
        return new List<string[]>() { item }
                .Union(
                    data
                        .Select(x => new string[] { x.Date.ToString("dd.MM.yyyy"), x.CountIn?.ToString("N0") ?? string.Empty, x.CountOut?.ToString("N0") ?? string.Empty }))
                .Union(
                    [["Всего", data.Sum(x => x.CountIn ?? 0).ToString("N0"), data.Sum(x => x.CountOut ?? 0).ToString("N0")]])
                .ToList();
    }
}
